
## 2011-Feb-07 
## (1) print-out format 
## (2) Statistics: z-approximation of the Chi-square 
##                 the exact binomial test 
##                 the Q-statistics

######################################################################################## 
## To consider use design matrix, instead of configuration patters, as input in cfa, 
## I revised the original functions cfa() and scfa(), and save the revised functions as 
##  cfa2() and scfa2()   
##  the key part is to use loglinear model to calculate the expected count values
## revised by Xianming TAN, 2010-Sep-23; 2011-Feb-07; 2011-April-17; 2011-Oct-01  
######################################################################################## 

#################
cfa2 <- function (cfg, conf_cols = NA, cnts = NA, sorton = "chisq", sort.descending = FALSE, 
    format.labels = TRUE, casewise.delete.empty = TRUE, binom.test = FALSE, 
    exact.binom.test = FALSE, exact.binom.limit = 10, perli.correct = FALSE, 
    lehmacher = FALSE, lehmacher.corr = TRUE, alpha = 0.05, bonferroni = TRUE) 
{
## 2011-Oct-01; add conf_cols = c(2:3) or NA to represent configuration matrice columns 
## cfg = X;  cnts= ofreq; binom.test=TRUE; sorton="label"; sort.descending=FALSE; alpha=0.05; bonferroni=TRUE;
##     format.labels = TRUE; casewise.delete.empty = TRUE;  
##     exact.binom.test = FALSE; exact.binom.limit = 10; perli.correct = FALSE; 
##     lehmacher = FALSE; lehmacher.corr = TRUE;

## cfg = X1; conf_cols=c(2:3);  cnt = ofreq1 ; binom.test=TRUE; sorton="label"; sort.descending=TRUE; alpha=0.05;

    bin.coeff <- function(n, k) exp(lgamma(n + 1) - lgamma(k + 
        1) - lgamma(n - k + 1))
    bin.test <- function(ni, ep, ntotal) {
        sum.p <- 0
        for (j in ni:ntotal) {
            sum.p <- sum.p + ((ep^j) * (1 - ep)^(ntotal - j) * 
                bin.coeff(ntotal, j))
        }
        ##sum.p
        sum.p = min(sum.p, 1-sum.p) ## use value close to 0; 2011-April-17
	  sum.p
    }
    z.test <- function(ni, ep, ntotal) { 
        num <- (ni - (ntotal * ep))
        den <- sqrt(ntotal * ep * (1 - ep))
        if ((ni[1] * ep[1]) <= 10) 
            num <- num - 0.5
        num/den
    }

    q.coef <- function(ni, ei, ntotal, T) {   ## 2011-Feb-07 by XTan
		## T is the number of cells, (? or ncases ? )
	  adj = c(0, 0.414, 0.578, 0.682, 0.741, 0.782, 0.811, 0.834, 0.852, 0.866, 
			seq(0.866, 0.910, length=6)[2:6], 
			seq(0.910, 0.932, length=6)[2:6], 
			seq(0.932, 0.972, length=31)[2:31], 
			seq(0.972, 0.982, length=26)[2:26], 
			seq(0.982, 0.986, length=26)[2:26]) 
	   if (T>100) T=100; 
	   if (T<1) T=1;   
	  Q_adj = adj[T]; ## will change this 
        num <- abs(ni - ei)
        den <- ei*(2*ei >= ni) + pmax(ei, ntotal-ei)*(2*ei < ni)*Q_adj
        num/den
    }

	## tmp = design; design = cfg; cfg = tmp;  ### add design matrix 


    if (is.null(dim(cfg))) 
        stop("cfg is probably part of a matrix! Use cfg[,1:n] rather than cfg[1:n]!")
    if (is.na(cnts)[1]) 
        cnts <- rep(1, nrow(cfg))
    if (length(cnts) == 1 && !is.data.frame(cnts)) 
        stop("cnts is probably part of a matrix! Use cnts[,n] rather than cnts[n]!")
    if (casewise.delete.empty) {
        cfg <- as.data.frame(cfg)
        idx <- rep(TRUE, nrow(cfg))
        for (i in 1:ncol(cfg)) {
            interim <- idx & (factor(cfg[, i]) != "") & (!is.na(cfg[, 
                i]))
            idx <- interim
        }
        cfg <- cfg[idx, ] ## matrix(cfg[idx, ], nrow = nrow(cfg))  ## cfg[idx, ]
        if (is.null(dim(cnts))) {
            cnts <- cnts[idx]
            ncnts <- 1
        }
        else {
            cnts <- cnts[idx, ]
            ncnts <- ncol(cnts)
        }
    }

     ## ncases <- nrow(cfg)
	if (!is.null(dim(cfg))) ## ordinary case 
	  {
		  ncases = nrow(aggregate(rep(1, nrow(cfg)), by = cfg, sum, na.rm = TRUE ) )
	  }
	if (is.null(dim(cfg)))   ## all 1 design matrix  
	  {
		cfg = matrix(cfg, nrow=length(cfg))
		ncases <- nrow(cfg)   ## add this (2011-Feb-06)
	  }
	
    ntotal <- sum(cnts, na.rm = TRUE)
    if (!is.null(nrow(cnts)) && (nrow(cfg) != nrow(cnts))) 
        stop("Configurations and counts must have the same numer of cases!")
    nconfigs <- ncol(cfg)
    n <- sum(cnts, na.rm = TRUE)
    multicfa <- FALSE
    if (!is.null(dim(cnts))) 
        multicfa <- TRUE
    if (multicfa == FALSE) {
        res <- scfa2(cfg, conf_cols, cnts, sorton, sort.descending, format.labels)
        bivariate <- TRUE
        if (any(res$n.levels > 2)) 
            bivariate <- FALSE
        configs <- length(res$chisq)
        if (bivariate == TRUE) {
            df <- (2^nconfigs - nconfigs - 1)
            dfcell <- 1
        }
        if (bivariate == FALSE) {
            df <- prod(res$n.levels) - sum(res$n.levels) + (nconfigs - 
                1)
            dfcell <- prod(res$n.levels - 1)
        }
        if (bivariate == TRUE) 
            ##z <- z.test(res$counts, res$expected/n, n)
            z <- (res$counts - res$expected) /sqrt(res$expected) ## 2011-Feb-07 by XTan
        else z <- sqrt(res$chisq)
        pz <- 1 - pnorm(abs(z)) ## revised April 17, 2011, use abs(z) instead of z
        #pz <- 1 - pnorm(z)
        nz <- length(z)
        if (bonferroni == FALSE) 
            nz <- 1
        pzsig <- ((1 - pnorm(abs(z), 0, 1)) < (alpha/nz) | (pnorm(abs(z), 0, 1) < (alpha/nz))) ## revised April 17, 2011
        #pzsig <- ((1 - pnorm(z, 0, 1)) < (alpha/nz) | (pnorm(z, 0, 1) < (alpha/nz)))
        sig.chisq <- (0.5*(1 - pchisq(res$chisq, dfcell)) ) < (alpha/nz)  ## April 17, 2011, add 0.5*...
        #sig.chisq <- (1 - pchisq(res$chisq, dfcell)) < (alpha/nz)
        totalchisq <- sum(res$chisq, na.rm = TRUE)
        p.total.chisq <- 1 - pchisq(totalchisq, dfcell)
        if (binom.test == TRUE) {
            z.bin <- (res$counts - res$expected)/sqrt(res$expected * 
                (1 - res$expected/n))
		p.z.bin = 1 - pnorm(abs(z.bin));  ## added on 2011-April-17;  
            #z.bin <- (res$counts - res$expected - 0.5*(res$expected>5 & res$expected<=10))/sqrt(res$expected * 
            #    (1 - res$expected/n))  ## 2011-Feb-07 by XTan; 2011-April-17 add >5 
            low.limit <- qnorm(alpha/2)
            hi.limit <- qnorm(1 - alpha/2)
            sig.bin <- ((z.bin < low.limit) | (z.bin > hi.limit))
        }
        if (exact.binom.test == TRUE && bivariate == TRUE) {
            p.exact.bin <- rep(NA, ncases)
            for (i in 1:ncases) {
                if (res$counts[i] <= exact.binom.limit) 
                  p.exact.bin[i] <- bin.test(res$counts[i], res$expected[i]/n, 
                    n)
            }
        }
        if (perli.correct == TRUE) {
            numerator.corr <- rep(0, length(res$counts))
            idx <- ((res$counts > 5) & (res$counts <= 10) & (res$counts - 
                res$expected > 0))
            numerator.corr[idx] <- (-0.5)
            idx <- ((res$counts > 5) & (res$counts <= 10) & (res$counts - 
                res$expected <= 0))
            numerator.corr[idx] <- 0.5
            x.perli <- (res$counts - res$expected + numerator.corr)/sqrt(res$expected)
            low.limit <- qnorm(alpha/2)
            hi.limit <- qnorm(1 - alpha/2)
            sig.perli <- ((x.perli < low.limit) | (x.perli > 
                hi.limit))
        }
        if (lehmacher == TRUE) {
            p18 <- apply((res$sums - 1), 1, prod)/((n - 1)^nconfigs)
            pn <- apply(res$sums, 1, prod)/(n^nconfigs)
            vl <- n * pn * (1 - pn - (n - 1) * (pn - p18))
            zl <- (res$counts - res$expected)/sqrt(vl)
            corr <- rep(0, length(res$counts))
            idx <- res$counts > res$expected
            corr[idx] <- -0.5
            idx <- res$counts < res$expected
            corr[idx] <- 0.5
            zl.corr <- (res$counts - res$expected + corr)/sqrt(vl)
            low.limit <- qnorm(alpha/2)
            hi.limit <- qnorm(1 - alpha/2)
            sig.zl <- ((zl < low.limit) | (zl > hi.limit))
            sig.zl.corr <- ((zl.corr < low.limit) | (zl.corr > 
                hi.limit))
        }
        #q <- 2 * abs(res$counts - res$expected)/(n + abs(2 * 
        #    res$expected - n))
        q <- q.coef(res$counts, res$expected, n, ncases) ## 2011-Feb-07 by XTan, T = ncases to be verified 
        resdataframe <- data.frame(label = res$labels, n = res$counts, 
            expected = res$expected, Q = q, chisq = res$chisq, 
            p.chisq = 0.5*(1 - pchisq(res$chisq, dfcell)), sig.chisq = sig.chisq,  ## add 0.5* (04-11-2011)
            ##p.chisq = 1 - pchisq(res$chisq, dfcell), sig.chisq = sig.chisq, 
            z = z, p.z = pz, sig.z = pzsig)
        if (perli.correct == TRUE) 
            resdataframe <- cbind(resdataframe, x.perli, sig.perli)
        if (lehmacher == TRUE && lehmacher.corr == FALSE) 
            resdataframe <- cbind(resdataframe, zl, sig.zl)
        if (lehmacher == TRUE && lehmacher.corr == TRUE) 
            resdataframe <- cbind(resdataframe, zl.corr, sig.zl.corr)
        if (binom.test == TRUE) 
            resdataframe <- cbind(resdataframe, z.bin, p.z.bin, sig.bin)
        if (exact.binom.test == TRUE && bivariate == TRUE) {
			resdataframe <- cbind(resdataframe, p.exact.bin)
			# if exact p-values are provided, base sig.bin decision on these values ...
			resdataframe$exact.sig.bin <- ifelse(resdataframe$p.exact.bin <= .05, "TRUE", "FALSE")
		}
        resvector <- c(totalchisq, df, p.total.chisq, n)
        names(resvector) <- c("totalchisq", "df", "p", "sum of counts")
        reslevels <- res$n.levels
        names(reslevels) <- colnames(cfg)
        res <- list(table = resdataframe, summary.stats = resvector, 
            levels = reslevels)
        class(res) <- "scfa2"
    }
    if (multicfa == TRUE) {
        res <- mcfa(cfg, cnts, sorton, sort.descending, format.labels)
        bivariate <- TRUE
        if (max(dim(table(cfg))) > 2) 
            bivariate <- FALSE
        configs <- length(res$chisq)
        counts <- ncol(cnts)
        factorn <- dim(table(cfg))
        if (bivariate == TRUE) {
            df <- (2^configs - 1) * (counts - 1)
            dfcell <- 1
        }
        if (bivariate == FALSE) {
            df <- prod(factorn) + sum(-factorn) + (configs - 
                1) * (counts - 1)
            dfcell <- prod(factorn - 1)
        }
        p.chisq <- 1 - pchisq(res$chisq, dfcell)
        nz <- length(p.chisq)
        if (bonferroni == FALSE) 
            nz <- 1
        pchisqsig <- (p.chisq < (alpha/nz))
        totalchisq <- sum(res$chisq, na.rm = TRUE)
        p.total.chisq <- 1 - pchisq(totalchisq, dfcell)

        resdataframe <- data.frame(label = res$labels, n = res$counts, 
            expected = res$expected, chisq = res$chisq, p.chisq = p.chisq, 
            p.chisq.sig = pchisqsig)
        resvector <- c(totalchisq, df, p.total.chisq, n)
        names(resvector) <- c("totalchisq", "df", "p", "sum of counts")
        res <- list(table = resdataframe, summary.stats = resvector)
        class(res) <- "mcfa"
    }
    res
}

print.scfa2 <- function(x, ...) {
	cat("\n*** Analysis of configuration frequencies (CFA) ***\n\n")
    print(x$table, digits=3)
    cat("\n\n")
    cat("Summary statistics:\n\n")
    cat("Total Chi squared         = ",x$summary.stats["totalchisq"],"\n") 
    cat("Total degrees of freedom  = ",x$summary.stats["df"],"\n") 
    cat("p                         = ",x$summary.stats["p"],"\n") 
    cat("Sum of counts             = ",x$summary.stats["sum of counts"],"\n")
    #cat("\n")
    #cat("Levels:\n\n")
    #print(x$levels,...)
    #cat("\n")
}


scfa2 <-function (cfg, conf_cols, cnt = NA, sorton = "chisq", sort.descending = TRUE, 
    format.labels = TRUE) 
{
## 2011-Oct-01, add parameter conf_cols 
## cnt = cnts ; dgn  design matrix 
##  res <- scfa2(cfg, cnts, sorton, sort.descending, format.labels)

## cfg = X;  conf_cols = NA; cnt = ofreq; sorton = "label"; sort.descending=TRUE; format.labels

    if (is.na(cnt)[1]) 
        cnt <- as.vector(rep(1, nrow(cfg)))
    if (!is.data.frame(cfg)) 
        cfg <- as.data.frame(cfg)

    if (!is.data.frame(cnt)) 
        cnt <- as.data.frame(cnt)
    cnt <- unlist(cnt)
    names(cnt) <- NULL

    d <- aggregate(cnt, by = cfg, sum, na.rm = TRUE)
    counts <- d[, ncol(d)]
    n <- sum(cnt)
    configs <- ncol(cfg)

    if (configs < 1) ## (configs < 2) 
        stop("At least two configuration variables required!")
    if (format.labels == TRUE) {
        for (i in 1:configs) levels(d[, i]) <- format(levels(d[, 
            i]))
    }
    sums <- matrix(data = NA, nrow = nrow(d), ncol = configs)
    n.levels <- vector(mode = "numeric", length = configs)

    if (configs >1)	
	{
    		string <- do.call("paste", d[, 1:configs])
	      strings <- strsplit(string, "\" *\"*")
		
		if ( is.null(length(conf_cols)) ) { dgn_mat <- strings  }    # added 2011-Oct-01
			else {
					d_tmp <- d[, conf_cols] 
					d_tmp[ d_tmp == -1  ] <- 2
					d_tmp[ d_tmp == 1  ] <- 1
			    		d_string <- do.call("paste", d_tmp)
	      			dgn_mat <- strsplit(d_string, "\" *\"*")
					strings <- dgn_mat  ## Nov 2, 2011 
				}
	}
	else 
	{
    		string <- rep("1", length(cnt))  ## ("paste", d[, 1:configs])
	      strings <- strsplit(string, "\" *\"*")

		dgn_mat <- strings   # added 2011-Oct-01 
      }

    for (i in 1:configs) { # i=1
        if (!is.factor(d[, i])) 
            d[, i] <- as.factor(d[, i])
        tbl <- aggregate(counts, d[i], sum)
        n.levels[i] <- nrow(tbl)
        dimnames(tbl)[1] <- tbl[1]
        if (dimnames(tbl)[1][[1]][1] == "") 
            dimnames(tbl)[1][[1]][1] <- "NA"
        cidx <- as.character(d[, i])
        cidx[cidx == ""] <- "NA"
        sums[, i] <- tbl[cidx, 2]
    }
    ## expected <- apply(sums, 1, prod)/n^(configs - 1)  ## only change this part, use log-linear model if teh input is a design matrix 
    ## expected <- (glm.fit(cfg, counts , family = poisson() ))$fitted.value  ## fitted.values;  ## only change this part, use log-linear model if teh input is a design matrix 
	dd=NULL; 
	for (i in 1:configs)
	{
		dd = cbind(dd,  as.numeric(levels(d[,i])[as.integer(d[,i])]))
	}
    if (configs >1)
	{	
	    expected <- (glm.fit(dd, counts, family = poisson(), intercept = FALSE ))$fitted.value  ## fitted.values;  ## only change this part, use log-linear model if teh input is a design matrix 
	}
	else  ## for cfg = rep(1, ...)
	{
	  expected = rep(mean(cnt), dim(cfg)[1] );
	  counts = cnt; 
	  sums = matrix(rep(sum(cnt), dim(cfg)[1]), ncol=1 )	
	}

    chisq <- (counts - expected)^2/expected
    sortidx <- order(chisq)
    if (sorton == "chisq") 
        sortidx <- order(chisq)
    if (sorton == "n") 
        sortidx <- order(counts)
    if (sorton == "label") 
        sortidx <- order(unlist(strings))
    if (sort.descending == TRUE) 
        sortidx <- rev(sortidx)
    colnames(sums) <- names(cfg)
    rownames(sums) <- strings

    # reslist <- list(labels = unlist(strings[sortidx]),  n.levels = n.levels, 
    reslist <- list(labels = unlist(dgn_mat[sortidx]),  n.levels = n.levels, 
        sums = sums[sortidx, ], counts = counts[sortidx], expected = expected[sortidx], 
        chisq = chisq[sortidx])
    reslist
}

### 

###